/**************************************************************************************
 **                                                                                  **
 **   Le Converleeter, créé par Moonlight Dreamer.                                   **
 **                                                                                  **
 **   ----------------------------------------------------------------------------   **
 **                                                                                  **
 **   Copyright (c) 2011 ANDRE Guillaume                                             **
 **   All rights reserved.                                                           **
 **                                                                                  **
 **   Redistribution and use in source and binary forms, with or                     **
 **   without modification, are permitted provided that the following                **
 **   conditions are met:                                                            **
 **                                                                                  **
 **   Redistributions of source code must retain the above copyright                 **
 **   notice, this list of conditions and the following disclaimer.                  **
 **   Redistributions in binary form must reproduce the above copyright              **
 **   notice, this list of conditions and the following disclaimer in                **
 **   the documentation and/or other materials provided with the distribution.       **
 **   Neither the name of the copyright holder nor the names of its                  **
 **   contributors may be used to endorse or promote products derived from           **
 **   this software without specific prior written permission.                       **
 **                                                                                  **
 **   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS            **
 **   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT              **
 **   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS              **
 **   FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT       **
 **   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,          **
 **   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED       **
 **   TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,            **
 **   OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY         **
 **   OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING        **
 **   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,   **
 **   EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.                             **
 **                                                                                  **
 **************************************************************************************/


#include "FenPersonnalisation.h"

using namespace std;

QSettings settings("MoonlightDreamer", "Converleeter");

FenPersonnalisation::FenPersonnalisation(bool *modifie, QWidget *parent)
     : QMainWindow(parent)
{
    setupUi(this);
    setWindowIcon(QIcon("IMAGES/logo.png"));

    m_nouveau->setIcon(QIcon("IMAGES/nouveau.png"));
    m_ouvrir->setIcon(QIcon("IMAGES/ouvrir.png"));
    m_enregistrer->setIcon(QIcon("IMAGES/enregistrer.png"));
    m_enregistrerSous->setIcon(QIcon("IMAGES/enregistrersous.png"));
    m_precisions->setIcon(QIcon("IMAGES/precisions.png"));

    QString texteCases("ABCDEFGHIJKLMNOPQRSTUVWXYZ");
    QString const texteCases2("ABCDEFGHIJKLMNOPQRSTUVWXYZ");
    m_modifie = modifie;
    m_mettreEtoile = true;
    m_fermerFenetre = false;

    m_nomFichierBas = new QLabel("Aucun fichier ouvert.");
    m_messageBas->addWidget(m_nomFichierBas);

    for(unsigned char i(0) ; i < 26 ; i++)
      m_alphabet[i] = new QLineEdit(settings.value(texteCases2[i]).toString() + "");


    QFormLayout *layoutCases[4];
    QGridLayout *layoutCases2 = new QGridLayout;

    for(unsigned char i(0) ; i < 4 ; i++)
      layoutCases[i] = new QFormLayout;

    for(unsigned char i(0) ; i < 7 ; i++)
      layoutCases[0]->addRow("&" + texteCases[i] + " : ", m_alphabet[i]);
    for(unsigned char i(7) ; i <= 13 ; i++)
      layoutCases[1]->addRow("&" + texteCases[i] + " : ", m_alphabet[i]);
    for(unsigned char i(14) ; i <= 20 ; i++)
      layoutCases[2]->addRow("&" + texteCases[i] + " : ", m_alphabet[i]);
    for(unsigned char i(21) ; i < 26 ; i++)
      layoutCases[3]->addRow("&" + texteCases[i] + " : ", m_alphabet[i]);

    for(unsigned char i(0) ; i < 4 ; i++)
      layoutCases2->addLayout(layoutCases[i], 0, i);

     m_layoutHaut->addLayout(layoutCases2);

    for(unsigned char i(0) ; i < 26 ; i++)
      connect(m_alphabet[i], SIGNAL(textEdited(QString)), this, SLOT(texteChange()));
}


FenPersonnalisation::~FenPersonnalisation()
{

}


void FenPersonnalisation::closeEvent(QCloseEvent *event)
{
    if(!*m_modifie)
    {
        if(m_nomFichierBas->text() == "Aucun fichier ouvert." || m_nomFichierBas->text().right(9) == "(modifié)")
        {
            if(QMessageBox::warning(this, "Attention", "Vous êtes sur le point de quitter et vous<br/>"
               "n'avez pas enregistré votre présentation.<br /><br />Voulez-vous vraiment quitter ?",
               QMessageBox::Yes | QMessageBox::No) == QMessageBox::No)
              event->ignore();
        }

        else
        {
            if(QMessageBox::warning(this, "Attention", "Vous n'avez pas validé votre présentation, "
               "voulez-vous vraiment quitter ?", QMessageBox::Yes | QMessageBox::No) == QMessageBox::No)
              event->ignore();
        }
    }
}


void FenPersonnalisation::on_m_nouveau_triggered()
{
    for(unsigned char i(0) ; i < 26 ; i++)
      m_alphabet[i]->clear();

    m_nomFichierBas->setText("Aucun fichier ouvert.");
    m_chemin = "";
}


void FenPersonnalisation::on_m_ouvrir_triggered()
{
    m_chemin = QFileDialog::getOpenFileName(this, "Ouverture du fichier", QString());
    ifstream flux(m_chemin.toStdString().c_str());

    if(flux)
    {
        string lettres("");

        for(unsigned char i(0) ; i < 26 ; i++)
        {
            if(flux.eof())
              m_alphabet[i]->clear();
            else
            {
                getline(flux, lettres);
                m_alphabet[i]->setText(QString::fromStdString(lettres));
            }
        }

        if(!trouverNomFichier().isEmpty())
          m_nomFichierBas->setText("Fichier ouvert : " + trouverNomFichier());
    }

    else if(!flux && !m_chemin.isEmpty())
      QMessageBox::critical(this, "Erreur", "Un problème a été rencontré, impossible d'ouvrir le fichier.<br />"
      "Veuillez quitter le logiciel et réessayer.");
}


void FenPersonnalisation::on_m_enregistrer_triggered()
{
    ofstream flux(m_chemin.toStdString().c_str());

    if(flux)
    {
        if(!m_chemin.toStdString().empty())
        {
            for(unsigned char i(0) ; i < 26 ; i++)
            {
                if(m_alphabet[i]->text().isEmpty())
                  flux << '\n';
                else if(i == 0)
                  flux << m_alphabet[i]->text().toStdString();
                else if(i > 0)
                  flux << '\n' + m_alphabet[i]->text().toStdString();
            }
        }

        if(!trouverNomFichier().isEmpty())
        {
            m_nomFichierBas->setText("Fichier ouvert : " + trouverNomFichier());
            m_mettreEtoile = true;
        }
    }

    else if(m_chemin.isEmpty())
      on_m_enregistrerSous_triggered();
    else if(!m_chemin.isEmpty() && !flux)
      QMessageBox::critical(this, "Erreur", "Un problème a été rencontré, impossible d'enregistrer le fichier.<br />"
      "Veuillez quitter le logiciel et réessayer.");
}


void FenPersonnalisation::on_m_enregistrerSous_triggered()
{
    m_chemin = QFileDialog::getSaveFileName(this, "Enregistrement du fichier", QString());
    ofstream flux(m_chemin.toStdString().c_str());

    if(flux)
    {
        for(unsigned char i(0) ; i < 26 ; i++)
        {
            if(m_alphabet[i]->text().isEmpty())
              flux << '\n';
            else if(i == 0)
              flux << m_alphabet[i]->text().toStdString();
            else if(i > 0)
              flux << '\n' + m_alphabet[i]->text().toStdString();
        }

        if(!trouverNomFichier().isEmpty())
          m_nomFichierBas->setText("Fichier ouvert : " + trouverNomFichier());
    }

    else if(!m_chemin.isEmpty() && !flux)
      QMessageBox::critical(this, "Erreur", "Un problème a été rencontré, impossible d'enregistrer sous"
      " le fichier.<br />Veuillez quitter le logiciel et réessayer.");
}


void FenPersonnalisation::on_m_precisions_triggered()
{
    QDialog *fenetre = new QDialog(this);
      fenetre->setWindowTitle("Précisions");
      fenetre->setWindowIcon(QIcon("IMAGES/precisions.png"));

    QVBoxLayout *layout = new QVBoxLayout;

    QLabel *label = new QLabel("Pour la fonction ouvrir, il faut qu'il y ait <strong>un saut de ligne entre<br />"
    "chaque lettre</strong> pour que le fichier à ouvrir soit lu correctement.<br />"
    "<u>Ex :</u><br /><br />/-\\<br />8<br />(<br />etc...<br /><br />"
    "Pour que votre style d'écriture soit considéré comme valide,<br />il faut que vous cliquiez sur valider..");

    layout->addWidget(label);
    fenetre->setLayout(layout);
    fenetre->exec();
}


void FenPersonnalisation::on_m_reinitialiser_clicked()
{
    for(unsigned char i(0) ; i < 26 ; i++)
      m_alphabet[i]->clear();

    if(!m_chemin.isEmpty())
      m_nomFichierBas->setText(m_nomFichierBas->text() + " (modifié)");
}


void FenPersonnalisation::on_m_annuler_clicked()
{
    *m_modifie = false;
    m_fermerFenetre = true;
    close();
}


void FenPersonnalisation::on_m_valider_clicked()
{
    char casesVides(0);

    for(unsigned char i(0) ; i < 26 ; i++)
    {
        if(m_alphabet[i]->text().isEmpty())
          casesVides++;
    }

    if(casesVides == 1)
    {
        QMessageBox::critical(this, "Erreur", "Un case est vide, veuillez la remplir.");
        return;
    }

    else if(casesVides > 1)
    {
        QMessageBox::critical(this, "Erreur", "Plusieurs cases sont vides, veuillez les remplir.");
        return;
    }

    else if(!verifierCases())
    {
        QMessageBox::critical(this, "Erreur", "Plusieurs cases sont identiques.<br />"
        "Il faut que toutes les cases soit différentes, sinon, il risquerait d'y avoir des problèmes.");
        return;
    }

    else if(casesVides == 0 && verifierCases())
    {
        if(m_conserver->isChecked())
        {
            QString const texteCases("ABCDEFGHIJKLMNOPQRSTUVWXYZ");

            for(unsigned char i(0) ; i < 26 ; i++)
              settings.setValue(texteCases[i], m_alphabet[i]->text());
        }

        *m_modifie = true;
        m_fermerFenetre = true;
        close();
    }
}


QString FenPersonnalisation::trouverNomFichier() const
{
    QString nomFichier("");
    unsigned int NombreSlashs(0);

    // On compte le nombre de slashs.
    for(int i(0) ; m_chemin[i] != '\0' ; i++)
    {
        if(m_chemin[i] == '/')
          NombreSlashs = i;
    }

    // On garde les lettres à partir du dernier slash.
    for(int i(NombreSlashs + 1) ; i < m_chemin.size() ; i++)
      nomFichier += m_chemin[i];

    return nomFichier;
}


bool FenPersonnalisation::verifierCases() const
{
    unsigned char casesIdentiques(0);

    for(unsigned char i(0) ; i < 26 ; i++)
    {
        casesIdentiques = 0;

        for(unsigned char j(0) ; j < 26 ; j++)
        {
            if(m_alphabet[i]->text() == m_alphabet[j]->text())
              casesIdentiques++;
        }

        if(casesIdentiques > 1)
          return false;
    }

    return true;
}


void FenPersonnalisation::texteChange()
{
    if(!m_chemin.isEmpty() && m_mettreEtoile)
    {
        m_nomFichierBas->setText(m_nomFichierBas->text() + " (modifié)");
        m_mettreEtoile = false;
    }
}


QLineEdit **FenPersonnalisation::getAlphabet()
{
    return m_alphabet;
}














